// Source file: c:/DopeWars/Framework/Connector.java

package DopeWars.Framework;

import java.io.*;
import java.net.*;

/**
 * This class is the connector used by a class that wants to open a single
 * connection to a MultiConnector.  This allows for two way communication.
 * A class implementing this method must also implement the IConnectorUser
 * interface so that the Connector may communicate back with it.
 *
 * @author Jordan Reed
 * @see IConnectorUser
 * @version 1.0
 * @since Phase 1.0 (Framework)
*/
public class Connector implements IConnector {
		/**
		 * This holds a reference to the inputStream that the class will use to
		 * read in information from the server.
		 *
		 * @since Phase 1.0 (Framework)
		*/
    private InputStream m_inputStream;

    /**
     * This holds a reference to the outputStream that the class will use to
     * send information out to the server.
     *
     * @since Phase 1.0 (Framework)
    */
    private OutputStream m_outputStream;

    /**
     * This holds a reference to the socket connecting this class to the
     * server side.  This reference should not be used past the constructor.
     *
     * @since Phase 1.0 (Framework)
    */
    private Socket m_socket;

    /**
     * This holds a reference to the object that instatated the connector.
     * It allows for the connector object to pass information back to the
     * caller.
     *
     * @see IConnectorUser
     * @since Phase 1.0 (Framework)
    */
    private IConnectorUser m_connectorUser;

    /**
     * This is the reference ID for this particular connector
     *
     * @since Phase 1.0 (Framework)
    */
    private String m_ID;

    /**
     * This holds a reference to the object that is listening
     *
     * @since Phase 1.0 (Framework)
     */
    private SocketListener m_socketListener;

    /**
     * This is public constructor.  It takes in an open socket as well as
     * a reference to the object that it should communicate calls to.  It
     * will use this open socket to create input and output streams for
     * communication with the connector on the other end.
     * <P>
     * It will also create a SocketThread which is responsible for
     * listening for new communication and passing information back to
     * the IConnectorUser.
     * <P>
     * This constructor will not assign an ID to the connector thus adding
     * this connector to a MultiConnector will throw an exception.
     *
     * @param socket Takes in the open socket to connect with the other side
     * @param connectorUser This holds a reference to the object that should get the response
     * @throws IOException
     * @since Phase 1.0 (Framework)
     * @roseuid 3925F8C90169
     */
		public Connector(Socket socket, IConnectorUser connectorUser)
		                 throws IOException {

			m_socket = socket;
			m_connectorUser = connectorUser;

			// All right now we make the Streams as well as the listener thread.
			m_inputStream = m_socket.getInputStream();
			m_outputStream = m_socket.getOutputStream();

			m_socketListener = new SocketListener(m_inputStream, this);
		}

    /**
     * This is public constructor.  It takes in an open socket as well as
     * a reference to the object that it should communicate calls to.  It
     * will use this open socket to create input and output streams for
     * communication with the connector on the other end.
     * <P>
     * It will also create a SocketThread which is responsible for
     * listening for new communication and passing information back to
     * the IConnectorUser.
     *
     * @param socket Takes in the open socket to connect with the other side
     * @param ID A unique identifier for the Connector user
     * @param connectorUser This holds a reference to the object that should get the response
     * @throws IOException
     * @since Phase 1.0 (Framework)
     * @roseuid 3925F8C90169
     */
    public Connector(Socket socket, String ID, IConnectorUser connectorUser)
                     throws IOException {

			this(socket, connectorUser);
			m_ID = ID;
    }

		/**
		 * This method will deconstruct a packet to its member parts and then pass
		 * them to the object it is listening for.
		 *
		 * @param packet The packet to be handled
		 * @since Phase 1.0 (Framework)
		 */

		public void handlePacket(Packet packet) {
			m_connectorUser.handleRequest(packet.getType(), packet.getThing(), m_ID);
		}

    /**
     * This get's called when the connector on the other end becomes
     * unreachable.  It needs to disband the socket listener and
     * then notify the owner of this connector that the connection is
     * dead.
     *
     * @since Phase 1.1 (Framework)
     * @roseuid 3925FCCB016F
     */
		public void closeConnection() {
			// This is basically the destructor.  So we'll set all unneeded
			// member variables to null and then call the chain of closes.
			m_inputStream = null;
			m_outputStream = null;
			m_socket = null;
			m_socketListener = null;

			m_connectorUser.closeConnection(m_ID);
		}

    /**
     * This method is used to send information over the stream to the other
     * side.  It simply creates a new packet with the output stream and the
     * rest is history.
     *
     * @param packetType The type of packet being send
     * @param thing An object to be sent with the packet
     * @see Packet
     * @since Phase 1.0 (Framework)
     * @roseuid 3925FCCB016F
     */
    public void send(int packetType, Object thing) {
			new Packet(m_outputStream, packetType, thing);
    }


  /**
   * This object listens for new packets being sent across the stream.
   * It will contact the listener when the new packet appears.
   */
	class SocketListener implements Runnable {
		/**
		 * Holds a reference to the input stream
		*/
		private InputStream m_inputStream;

		/**
		 * Holds a reference to the object to give the new packet to
		 */
    private Connector m_connector;

		/**
		 * Constructor takes in the input stream and object to talk with.
		 *
		 * @param inputStream The stream to listen to
		 * @param connector	The connect that will handle the packet
		 */
	  public SocketListener (InputStream inputStream, Connector connector) {
			m_inputStream = inputStream;
			m_connector = connector;

			System.out.println("Created new SocketListener");
			(new Thread(this, "Connector Listener")).start();
		}

	  public void run() {
			boolean connectionOkay = true;
			Packet tempPacket;

	    while(connectionOkay) {
				try {
					tempPacket = new Packet(m_inputStream);
					m_connector.handlePacket(tempPacket);
				} catch (IOException e) {
					System.out.println("IOException");
					// The packet has failed.  So this connection is dead.
					connectionOkay = false;
				}
			}
			m_connector.closeConnection();
		}
	}
}
