// Source file: c:/DopeWars/Framework/MultiConnector.java

package DopeWars.Framework;

import java.io.*;
import java.net.*;
import java.util.*;

/**
 * The multiconnector is established by the server to manage multiple
 * connectors which allow a server to attach to many clients.
 *
 * @author Jordan Reed
 * @see Connector
 * @see IConnectorUser
 * @version 1.0
 * @since Phase 1.0 (Framework)
*/
public class MultiConnector implements IConnectorUser {
	  /**
	   * This contains references to each connector managed by the multi-
	   * connector.
	   *
	   * @since Phase 1.0 (Framework)
	  */
    private HashMap m_connectorMap;

		/**
		 * This constains a number which is used to create a unique key
		 * to store the connectors against.
		 */
		private long keyValue = 0;

    /**
     * This stores a reference to the object using this multiconnector
     *
     * @since Phase 1.0 (Framework)
    */
    private IConnectorUser m_connectorUser;

    /**
     * This holds a reference to the thread that is listening for new
     * connection requests
     *
     * @since Phase 1.0 (Framework)
     */
    private ServerListenerThread m_serverListenerThread;

    /**
     * This constructor will store the connectorUser and spawn a thread that
     * will listen on the server socket and react each time a new connection
     * is requested.
     *
     * @param socket The server socket that needs to be listened on.
     * @param connectorUser The object that the connector should talk to.
     * @roseuid 3930538702F9
     */
    public MultiConnector(ServerSocket socket, IConnectorUser connectorUser) {
			System.out.println("MultiConnector created");
			m_connectorUser = connectorUser;
			m_connectorMap = new HashMap();
			m_serverListenerThread = new ServerListenerThread(this, socket);
    }

    /**
     * This sends a packet of information to the all receivers
     *
     * @param packetType The type of packet being sent
     * @param thing The object to be sent
     * @since Phase 1.0 (Framework)
     * @roseuid 3926024D01E4
     */
    public void send(int packetType, Object thing) {
			System.out.println("Sending Packet to all...");
			Connector tempConnector;
			Set tempSet = m_connectorMap.entrySet();
			Iterator connectorIter = tempSet.iterator();
			while(connectorIter.hasNext()) {
				Map.Entry me = (Map.Entry) connectorIter.next();
				tempConnector = (Connector) me.getValue();
				tempConnector.send(packetType, thing);
			}
    }

    /**
     * This sends a packet of information to the specified receiver
     *
     * @param packetType The type of packet being sent
     * @param thing The object to be sent
     * @param receiver The unique identifier of the receiver.  This will be used for lookup on m_connectorMap
     * @since Phase 1.0 (Framework)
     * @roseuid 3926024D01E4
     */
    public void send(int packetType, Object thing, String receiver) {
			Connector tempConnector = (Connector) m_connectorMap.get(receiver);
			tempConnector.send(packetType, thing);
    }

    /**
     * This will take in a socket and generate a new connector and add it
     * to the internal map of connectors.
     *
     * @param socket A socket to use with the connector
     * @since Phase 1.0 (Framework)
     * @roseuid 3926049903A3
     */
    public void addConnector(Socket socket) {
			Connector tempConnector;
			String tempKey;
			System.out.println("Adding Connector");

			// Generate a new key value and turn it into a string.
			tempKey = "" + (++keyValue);

			try {
				// First we generate a new Connector
				tempConnector = new Connector(socket, tempKey, this);

				System.out.println("Adding connector: " + tempKey);
				m_connectorMap.put(tempKey, tempConnector);
			} catch (IOException e) {
				System.out.println("Error making connector");
				keyValue--;
			}
    }

    /**
     * This method will just hand the request back to the object that is
     * attached to the multiconnector.
     *
     * @see IConnectorUser
     */
    public void handleRequest(int requestType, Object requestObject, String requestor) {
			m_connectorUser.handleRequest(requestType, requestObject, requestor);
		}

		/**
		 * This method will remove the connector from the multiconnector
		 * and notify the object that is attached to the muliconnector.
		 *
		 * @see IConnectorUser
		 */
		public void closeConnection(String requestor) {
			// Simply remove the connector from the table
			Object Connector = m_connectorMap.remove(requestor);
			System.out.println("Ditching " + requestor + " from MultiConnector");
			// And pass up the call
			m_connectorUser.closeConnection(requestor);
		}

		/**
		 * This class creates a thread which listens for new requests on the
		 * multiconnector.  Each time a new request is made it will add a new
		 * connector.
		 *
		 * @author Jordan Reed
		 * @version 1.0
		 * @since Phase 1.0 (Framework)
		 */
		private class ServerListenerThread implements Runnable {
			/**
			 * Holds a reference to the MultiConnector which will get the
			 * new connection.
			*/
			private MultiConnector m_multiConnector;

			/**
			 * The socket that clients will make requests on
			 */
			private ServerSocket m_serverSocket;

			/**
			 * The constructors stores the needed references and starts a thread
			 * from this class that will listen.
			 *
			 * @param mc The multiconnector this thread should talk with
			 * @param ss The server socket to listen on
			 */
			public ServerListenerThread(MultiConnector mc, ServerSocket ss) {
				System.out.println("ServerListenerThread created");
				m_multiConnector = mc;
				m_serverSocket = ss;
			  (new Thread(this, "Server Listner Thread")).start();
			}

			/**
			 * The main method of the thread.  This will listen for requests to
			 * the socket and then create ask the multiconnector to make a new
			 * connector on each new request.
			 *
			 */
			public void run() {
				Socket s;

				while(true) {
					try {
						System.out.println("Waiting for Connection");
						s = m_serverSocket.accept();
						System.out.println("Connection Establish");
						m_multiConnector.addConnector(s);
					} catch (IOException e) {
						System.out.println("Broke Attempting to make connection");
					}
				}
			}
		}
}
